// Author:   Kamaledin Ghiasi-Shirazi


#include <algorithm>
#include <vector>

#include "caffe/layer.hpp"
#include "caffe/layers/sin_layer.hpp"

namespace caffe {

template <typename Dtype>
void SinLayer<Dtype>::Forward_gpu(const vector<Blob<Dtype>*>& bottom,
    const vector<Blob<Dtype>*>& top) {
  const int count = bottom[0]->count();
  const Dtype* bottom_data = bottom[0]->gpu_data();
  Dtype* data_cos = data_cos_.mutable_gpu_data();
  Dtype* top_data = top[0]->mutable_gpu_data();
  caffe_gpu_sin(count, bottom_data, top_data);
  caffe_gpu_cos(count, bottom_data, data_cos);
}

template <typename Dtype>
void SinLayer<Dtype>::Backward_gpu(const vector<Blob<Dtype>*>& top,
    const vector<bool>& propagate_down,
    const vector<Blob<Dtype>*>& bottom) {
  if (!propagate_down[0]) { return; }
  const int count = bottom[0]->count();
  const Dtype* data_cos = data_cos_.gpu_data();
  const Dtype* top_diff = top[0]->gpu_diff();
  Dtype* bottom_diff = bottom[0]->mutable_gpu_diff();
  caffe_gpu_mul(count, data_cos , top_diff, bottom_diff);
}
INSTANTIATE_LAYER_GPU_FUNCS(SinLayer);

}  // namespace caffe
