// This program converts a set of images to a lmdb/leveldb by storing them
// as Datum proto buffers.
// Usage:
//   extract_image [FLAGS] DB_NAME OUT_FILE
//

#include <algorithm>
#include <fstream>  // NOLINT(readability/streams)
#include <string>
#include <utility>
#include <vector>

#include "boost/scoped_ptr.hpp"
#include "gflags/gflags.h"
#include "glog/logging.h"

#include "caffe/proto/caffe.pb.h"
#include "caffe/util/db.hpp"
#include "caffe/util/io.hpp"
#include "caffe/util/rng.hpp"

using namespace caffe;  // NOLINT(build/namespaces)
using std::pair;
using boost::scoped_ptr;


int main(int argc, char** argv) {
  ::google::InitGoogleLogging(argv[0]);

#ifndef GFLAGS_GFLAGS_H_
  namespace gflags = google;
#endif

  gflags::SetUsageMessage("Convert a set of images to the leveldb/lmdb\n"
        "format used as input for Caffe.\n"
        "Usage:\n"
        "    convert_imageset [FLAGS] ROOTFOLDER/ LISTFILE DB_NAME\n"
        "The ImageNet dataset for the training demo is at\n"
        "    http://www.image-net.org/download-images\n");
  gflags::ParseCommandLineFlags(&argc, &argv, true);

  if (argc < 3) {
    gflags::ShowUsageWithFlagsRestrict(argv[0], "tools/extract_image");
    return 1;
  }

  // Create new DB
  scoped_ptr<db::DB> db(db::GetDB("lmdb"));
  db->Open(argv[1], db::READ);
  shared_ptr<db::Cursor> cursor(db->NewCursor());
  db::Cursor* cursor2 =  cursor.get();
  Datum datum;
  datum.ParseFromString(cursor2->value());
  const std::string data = datum.data();

  fstream outfile;
  outfile.open (argv[2], ios::out);
  for (int i = 0; i < data.size(); i++)
  {
	outfile << (int)(unsigned char)data[i] << " ";
	if (i % 28 ==0)
		outfile << "\n";
  } 

  outfile.close();
  
 return 0;
}
